/*
 * connection.h
 *
 * Kai G. Schwebke
 * 2007-03-02
 *
 * Client connection handling.
 *
 */

#ifndef IncConnection
#define IncConnection
#include "connectionmanager.h"
#include <pthread.h>
#include <string>

class ConnectionManager;

// Connection to remote client and associated thread
// serving the connection.
class Connection {
   public:
      // wrap the given file descriptor fd and create
      // serving thread
      Connection(ConnectionManager* manager, int fd);

      // join the serving thread (it will terminate if
      // shutdownDaemon is flagged or remote side closed
      // the connection) and close connection.
      virtual ~Connection();

      // serving function called by serving thread -- do
      // not call directly
      void startServing();

   private:
      // process incoming data
      int process(std::string line);

      // associated serving thread
      pthread_t thread;

      // client connection file descriptor
      int fd;

      // manager for this connection
      ConnectionManager* manager;
};

// serving thread function
void* connectionThread(Connection*);

#endif
