/*
 * connectionmanager.h
 *
 * Kai G. Schwebke
 * 2007-03-02
 *
 * Listening for incoming connections.
 * Creates and manages Connection objects.
 *
 */
#ifndef IncConnectionManager
#define IncConnectionManager
#include <list>
#include <pthread.h>
#include "connection.h"

class Connection;
class SensorManager;

// The connecion manager listens for incoming
// connections, accepts and spawns serving
// threads for them.
// If shutdownDaemon is flagged the manager closes
// all connections and stops serving.
class ConnectionManager {
   public:
      // Create new manager. Serving is not started!
      ConnectionManager(SensorManager* sensorManager);

      // Close all connections and destroy manager.
      virtual ~ConnectionManager();

      // Start serving on listen port until shutdownDaemin
      // is flagged.
      void startListen(int listenPort);

      // Close connection and remove from list. This
      // method is used by a connection -- do not
      // call directly.
      void closeConnection(Connection* connection);

      inline SensorManager* getSensorManager() {
	 return sensorManager;
      }

   private:
      // Handle new connection.
      void newConnection(int fd);

      // Listen socket file descriptor.
      int listenSocketFd;

      // List of open connections.
      std::list<Connection*> connections;

      // Mutex used to synchronize access.
      pthread_mutex_t mutex;

      SensorManager* sensorManager;
};

#endif
