/*
 * daemon.cpp
 *
 * Kai G. Schwebke
 * 2007-03-02
 *
 * Daemon process creation.
 *
 */
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "logging.h"
#include "daemon.h"
using namespace std;


void daemonize() {
   // fork() a child, so the parent can exit.
   // This returns control to the invoking command line or shell.
   pid_t pid = fork();

   if (pid == -1) {
      logEvent(LOG_CRITICAL, "cannot fork session group leader");
      exit(3);
   }

   // sessiong group leader child
   if (pid == 0) {
      // setsid() to become a process group and session group leader
      // without controlling terminal.
      if (setsid() == -1) {
	 logEvent(LOG_CRITICAL, "setsid failed");
	 exit(3);
      }

      // fork() again so the parent (session group leader), can exit.
      // The child can never regain a controlling terminal.
      pid_t pid = fork();
      if (pid == -1) {
	 logEvent(LOG_CRITICAL, "cannot fork daemonized child");
	 exit(3);
      }

      // daemonized child
      if (pid == 0) {
	 // chdir("/") to ensure that our process doesn't keep
	 // any directory in use.
	 if (chdir("/")) {
	    logEvent(LOG_CRITICAL, "chdir root failed");
	    exit(3);
	 }

	 // Close all file descriptors (and ignore errors).
	 int i;
	 for (i=getdtablesize(); i>=0; --i) close(i);

	 // Assign null device to stdin, stdout and stderr.
	 // From now on, only the logging API should be used.
	 i=open("/dev/null", O_RDWR); // stdin
	 dup(i); // stdout
	 dup(i); // stderr

	 logEvent(LOG_DEBUG, "daemonized process");
      } else {
         // exit parent (using _exit, so no libc cleanup is performed).
	 _exit(0);
      }
   } else {
      // exit parent (using _exit, so no libc cleanup is performed).
      _exit(0);
   }
}
