/*
 * logging.cpp
 *
 * Kai G. Schwebke
 * 2007-03-02
 *
 * Message logging.
 *
 */
#include <cstdio>
#include <string>
#include <cstdlib>
#include <ctime>
#include <unistd.h>
#include <pthread.h>
#include "config.h"
#include "logging.h"
using namespace std;

string logFile("");

int logLevel                  = LOG_DEBUG;

// mutex to ensure thread safe logging
pthread_mutex_t logMutex;

// initialize logging
void logInit() {
   pthread_mutex_init(&logMutex, 0);
}

// shutdown logging
void logShutdown() {
   setLogLevel(LOG_DISABLED);
   pthread_mutex_destroy(&logMutex);
}

void setLogLevel(int newLevel) {
   pthread_mutex_lock(&logMutex);
   logLevel = newLevel;
   pthread_mutex_unlock(&logMutex);
}

// Log an event to log file (daemon mode) or console,
// if level is equal to or more severe than logLevel.
// The log file in daemon mode is opened and closed
// for every event, so it can be easily truncated without
// stopping the daemon.
void logEvent(int level, string message) {
   if (pthread_mutex_lock(&logMutex)) {
      fprintf(stderr, "cannot lock logMutex\n");
      exit(3);
   }
   if ( (level <= logLevel) && (level >= 0) ){
      FILE* log = 0;
      if (logFile == "") {
	 log = stdout;
      } else {
	 log = fopen(logFile.c_str(), "a");
	 if (!log) {
	    fprintf(stderr, "cannot open log file '%s'\n", logFile.c_str());
	    exit(2);
	 }
      }

      char timeStamp[20];
      struct tm* tmNow;
      time_t tNow = time(NULL);
      tmNow = localtime(&tNow);
      strftime(timeStamp, 20, "%Y-%m-%d %H:%M:%S", tmNow);

      fprintf(log, "%s - %s\n", timeStamp, message.c_str());

      if (logFile != "") {
	 fclose(log);
      }

   }
   if (pthread_mutex_unlock(&logMutex)) {
      fprintf(stderr, "cannot lock logMutex\n");
      exit(3);
   }
}

