/*
 * sensor.h
 *
 * Kai G. Schwebke
 * 2007-03-03
 *
 * Sensor data handling.
 *
 */
#ifndef IncSensor
#define IncSensor

#include <time.h>
#include <pthread.h>
#include <string>
#include <map>


class SensorManager;

class Sensor {
   public:
      // Create sensor handler for given serial device name
      Sensor(SensorManager* manager, std::string deviceName);

      // join handling thread (it will terminate if
      // shutdownDaemin is flagged) and close device.
      virtual ~Sensor();

      // Create sensor handling thread.
      void createThread();

      // handling function called by handling thread --
      // do not call directly
      void startSensing();

      // get serial device name
      std::string getDeviceName() {
	 return deviceName;
      }

      // type of sensed value
      enum ReadType {
	 TEMPERATURE = 1,
	 HUMIDITY    = 2,
	 DEWPOINT    = 3
      };

      // get last reading.
      // Returns "" if no valid reading
      // is available.
      std::string getReading(ReadType readType);

   private:
      // process incoming data
      void process(std::string line);

      // name of serial device sensor is attached
      std::string deviceName;

      // contents of last reading
      std::map<std::string, std::string> readContent;

      // time of last reading
      time_t readTime;

      // handling thread
      pthread_t thread;

      // Mutex used to synchronize access.
      pthread_mutex_t mutex;

      SensorManager* manager;

      // lookup table from ReadType to name as delivered from sensor
      static const int ReadNameCount;
      static const char * const ReadName[];
};

// handling thread
void* sensorThread(Sensor* s);

#endif
