/*
 * sensormanager.cpp
 *
 * Kai G. Schwebke
 * 2007-03-03
 *
 * Creation and naming of Sensor objects.
 *
 */
#include "sensormanager.h"

using namespace std;

SensorManager::SensorManager() {
   pthread_mutex_init(&mutex, 0);
}

SensorManager::~SensorManager() {
   pthread_mutex_lock(&mutex);

   list<Sensor*>::iterator i;
   for (i = sensors.begin(); i != sensors.end(); ++i) {
      delete *i;
   }

   pthread_mutex_unlock(&mutex);
   pthread_mutex_destroy(&mutex);
}

void SensorManager::createThreads() {
   pthread_mutex_lock(&mutex);

   list<Sensor*>::iterator i;
   for (i = sensors.begin(); i != sensors.end(); ++i) {
      (*i)->createThread();
   }

   pthread_mutex_unlock(&mutex);
}

Sensor* SensorManager::newSensor(string deviceName) {
   pthread_mutex_lock(&mutex);

   Sensor* s = new Sensor(this, deviceName);
   sensors.push_back(s);

   pthread_mutex_unlock(&mutex);
   return s;
}

void SensorManager::nameSensor(Sensor* s, string name) {
   pthread_mutex_lock(&mutex);

   sensorNameMap[name] = s;

   pthread_mutex_unlock(&mutex);
}

string SensorManager::getReading(string name) {
   string r = "";
   string::size_type pos = name.find_last_of('.');

   if (pos != string::npos) {
      pthread_mutex_lock(&mutex);

      string sensorName = name.substr(0, pos);
      string sensorType = name.substr(pos+1);

      if (sensorNameMap.count(sensorName) > 0) {
	 r = sensorNameMap[sensorName]->getReading((Sensor::ReadType)atoi(sensorType.c_str()));
      }

      pthread_mutex_unlock(&mutex);
   }

   return r;
}
