/*
 * serial.cpp
 *
 * Kai G. Schwebke
 * 2007-03-02
 *
 * Serial port configuration.
 *
 */
#include "serial.h"
#include <string>
#include <cstdio>
#include <cstring>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>

using namespace std;

Serial::Serial(string device) {
   this->device = device;
   this->fd = -1;
   open();
}

Serial::~Serial() {
   if (fd >= 0) {
      close(fd);
   }
}

void Serial::open() {
   // open device
   fd = ::open(device.c_str(), O_RDWR|O_NOCTTY);
   if (fd < 0) {
      return;
   }

   // set baud rate
   struct termios options;
   tcgetattr(fd, &options);
   cfsetispeed(&options, B9600);
   options.c_cflag |= (CLOCAL|CREAD);
   
   // no parity 8N1
   options.c_cflag &= ~PARENB;
   options.c_cflag &= ~CSTOPB;
   options.c_cflag &= ~CSIZE;
   options.c_cflag |= CS8;

   // no RTS/CTS flow control
   options.c_cflag &= ~CRTSCTS;

   // no XON/XOFF flow control
   options.c_iflag &= ~(IXON | IXOFF | IXANY);

   // raw input
   options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

   // raw output
   options.c_oflag &= ~OPOST;

   if (tcsetattr(fd, TCSANOW, &options)) {
      perror("tcsetattr failed");
   }
}

