/*
 * tool.cpp
 *
 * Kai G. Schwebke
 * 2007-03-03
 *
 * Helper functions.
 *
 */
#include <string>
#include <vector>
#include <sys/types.h>
#include <sys/socket.h>
#include "config.h"
#include "logging.h"

void stringTokenize(const std::string& str,
                    std::vector<std::string>& tokens,
                    const std::string& delimiters = " ")
{
    // Skip delimiters at beginning.
    std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    std::string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (std::string::npos != pos || std::string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}

std::string itoa(int i) {
   char buff[1024];
   snprintf(buff, 1024, "%d", i);
   return std::string(buff);
}


int shouldShutdown = 0;

// cleanup done on shutdown
void shutdown() {
   shouldShutdown = 1;
   logEvent(LOG_CRITICAL, "ending daemon");
   logShutdown();
   exit(0);
}


// show command line help
void showHelp() {
   printf("Usage: daemon [options] [config file]\n");
   printf("Options:\n");
   printf("   -d               Become a daemon\n");
   printf("   -h               Show this help\n");
   printf("Configuration is read from %s\n", DEFAULT_CONFIG_FILE);
   printf("or the given config file.\n");
}

// send data over socket file descriptor
int sendall(int s, const void* buff, size_t* len) {
   int total = 0;        // how many bytes we've sent
   int bytesleft = *len; // how many we have left to send
   int n;

   while(total < *len) {
      n = send(s, ((const char*)buff)+total, bytesleft, 0);
      if (n == -1) { break; }
      total += n;
      bytesleft -= n;
   }

   *len = total; // return number actually sent here

   return n==-1?-1:0; // return -1 on failure, 0 on success
}
